import java.util.List;
import java.util.Iterator;
import java.util.Random;

/**
 * A simple model of an elephant.
 * Animals age, move, and die. 
 * Elephants just wanders around, and they might crush other animals in their way.
 * If the elephant moves to a field where there is any other kind of animal, that animal dies.
 * 
 * @author Poul Henriksen
 * @version 2005.03.14
 */
public class Elephant extends Animal
{
    // Characteristics shared by all foxes (static fields).
    
    // The age at which a fox can start to breed.
    private static final int BREEDING_AGE = 80;
    // The age to which a fox can live.
    private static final int MAX_AGE = 100;
    // The likelihood of a fox breeding.
    private static final double BREEDING_PROBABILITY = 0.04;
    // The maximum number of births.
    private static final int MAX_LITTER_SIZE = 2;
    
    // Individual characteristics (instance fields).

    /**
     * Create an elephant. An elephant can be created as a new born (age zero
     * and not hungry) or with random age.
     * @param randomAge If true, the elphant will have random age and hunger level.
     */
    public Elephant(boolean randomAge)
    {
        super();
        if(randomAge) {
            setAge(rand.nextInt(MAX_AGE));
        }
    }
    
    /**
     * This is what the elephant does most of the time: it walks around.
     * In the process, it might breed, crush other animals, or die of old age.
     * @param currentField The field currently occupied.
     * @param updatedField The field to transfer to.
     * @param newAnimals A list to add newly born foxes to.
     */
    public void act(Field currentField, Field updatedField, List newAnimals)
    {
        incrementAge();
        if(isAlive()) {
            // New elephants are born into adjacent locations.
            int births = breed();
            for(int b = 0; b < births; b++) {
                Elephant newElephant = new Elephant(false);
                newAnimals.add(newElephant);
                newElephant.setLocation(
                        updatedField.randomAdjacentLocation(getLocation()));
                updatedField.place(newElephant);
            }
            // move randomly
            Location newLocation = updatedField.randomAdjacentLocation(getLocation());
            Animal animal = currentField.getObjectAt(newLocation);
            if(animal instanceof Elephant) {
                updatedField.place(this);  // sets location
            } else if (animal != null){
                animal.setDead();
                setLocation(newLocation);
                updatedField.place(this);  // sets location
            } else {
                setLocation(newLocation);
                updatedField.place(this);  // sets location
            }
        }
    }
    
    /**
     * @return A string representation of the elephant.
     */
    public String toString()
    {
        return "Elephant, age " + getAge();
    }
    
    /**
     * @return The age at which a elephant starts to breed
     */    
    protected int getBreedingAge() 
    {
            return BREEDING_AGE;
    }
            
    /**
     * @return The age at which a elephant dies.
     */       
    protected int getMaxAge()
    {
            return MAX_AGE;
    }
    
    /**
     * @return The likelihood of an animal breeding.
     */   
    protected double getBreedingProbability() 
    {
        return BREEDING_PROBABILITY;
    }
    
    /**
     * @return The maximum number of births.
     */
    protected int getMaxLitterSize()
    {
        return MAX_LITTER_SIZE;
    }
        
}
